<?php
/**	
 *
 *	PDF LETTER CLASS
 *	Author: Paul Hamlington
 *	Purpose: Control all communication with TCPDF. 
 * 
 */

    /** [ SET NAMESPACE: MODULIZED DOCSAWAY ] */
    namespace modulized_docsaway;	
	

    /** [ PDF LETTER CLASS ] */
    final class pdf_letter
        {
	    /** [ PRIVATE CLASS PROPERTIES ] */
	    private $globals;
        
		
	    /** --------------------------> CLASS INTERFACE METHODS <--------------------------*/
		
        
        /** [ METHOD: EXECUTE ] */
        public function setRecipient($arr){$this->globals['recipient']=$arr;}
        public function setContent($str){$this->globals['content']=$str;}
        
        /** [ METHOD: EXECUTE ] */
        public function execute(){return($this->executeFunc());}
        
        /** [ METHOD: GET PDF ] */
        public function getPDF(){return($this->getPDFFunc());}

        
	    /** --------------------------> CLASS FUNCTIONALITY METHODS <--------------------------*/
		
		      
	    /** [ METHOD: CONSTRUCTOR ] */
	    public function __construct()
            {
            /** [ DEFAULT: SETTING ] */
            $this->globals['orientation'] ='P';     /* [ Portrait ] */
            $this->globals['unit']        ='mm';    /* [ Millimeter ] */
            $this->globals['format']      ='A4';    /* [ A4 Papersize ] */
            $this->globals['unicode']     =true;    /* [ Character Encoding ] */
            $this->globals['encoding']    ='UTF-8'; /* [ Character Encoding ] */
            $this->globals['diskcache']   =false;   /* [ Caching temporary data on filesystem ] */
            $this->globals['pdfa']        =false;   /* [ PDF/A mode ] */
            $this->globals['html']        =false;
            $this->globals['pdf']         =false;
            $this->globals['sender']      =false;
            $this->globals['recipient']   =false;
            $this->globals['content']     =false;
            
            /** [ RETURN: BOOLEAN ] */
            return(true);
            }
            
        /** [ METHOD: IMPORT CLASS ] */	
        private function importClass($class)
            {
            /** [ CLASS SELECTION ] */
            switch($class)
                {
                /** [ IMPORTED OBJECTS ] */
                case 1: return(new \TCPDF($this->globals['orientation'],$this->globals['unit'],$this->globals['format'],$this->globals['unicode'],$this->globals['encoding'],$this->globals['diskcache'],$this->globals['pdfa']));
                case 2: return(new templates());
                case 3: return(new whmcs());
                }
            }
            
        /** [ METHOD: EXECUTE ] */    
        private function executeFunc()
            {     
            /** [ METHOD: CONFIGURATION ] */
            if($this->configuration()===false){return(false);} 
            
            /** [ METHOD: LETTER CONTENT ] */
            if($this->letterContent()===false){return(false);}
            
            /** [ METHOD: HTML STRUCTURE ] */
            $this->htmlStructure();
            
            /** [ METHOD: CREATE PDF ] */
            $this->createPDF();
            
            /** [ RETURN: BOOLEAN ] */
            return(true);
            } 
            
        /** [ METHOD: CONFIGURATION ] */    
        private function configuration()
            {
            /** [ OBJECT: WHMCS ] */
            $obj = $this->importClass(3);
            $file=$obj->getConfiguration();
            unset($obj);
            
            /** [ STORE: DETAILS ] */
            $this->globals['sender']['company'] =$file['companyname'];
            $this->globals['sender']['logo']    =$file['logourl'];
            $this->globals['sender']['address'] =(!empty($file['invoicepayto']) ?  preg_replace("/(\n|\r)+/",'<br/>',$file['invoicepayto']) : '');
           
            /** [ IMAGE: REQUIRED ] */
            if(!empty($this->globals['sender']['logo']))
                {
                /** [ ERROR: IMAGE FAILED TO LOAD ] */
                if(@getimagesize($this->globals['sender']['logo'])===false)
                    {
                    /** [ METHOD: DEBUGGER ] */
                    debugger::err('logo image failed to load('.$this->globals['sender']['logo'].')',__FILE__,__CLASS__,__METHOD__,__LINE__);
                    
                    /** [ RETURN: BOOLEAN ] */
                    unset($file); return(false);
                    }
                }
            
            /** [ RETURN: BOOLEAN ] */
            unset($file); return(true);
            }
                   
        /** [ METHOD: LETTER CONTENT ] */    
        private function letterContent()
            {
            /** [ METHOD: NAME ] */
            $name=globals::name($this->globals['recipient']['name']);

            /** [ TOKENS: VARIABLES ] */
            $tokens['first_name'] =$name['first'];            
            $tokens['company_name'] =$this->globals['sender']['company'];

            /** [ OBJECT: TEMPLATES ] */
            $obj=$this->importClass(2);
            $this->globals['content']=str_ireplace("\n",'<br/>',$this->globals['content']);
            $this->globals['content']=$obj->replaceTokens($this->globals['content'],$tokens);
            unset($obj);
            
            /** [ RETURN: BOOLEAN ] */
            unset($tokens,$name); return(true);
            }
                        
        /** [ METHOD: HTML STRUCTURE ] */    
        private function htmlStructure()
            {
            /** [ CONVERT: COUNTRY CODE ] */
            $this->globals['recipient']['country']=country::conv($this->globals['recipient']['country']);
            
            /** [ TEMPLATE: RECIPIENT FIELDS ] */
            $arr['fields']=array('name','company','address1','address2','city','state','postcode','country');
            
            /** [ BUILD: RECIPIENT ADDRESS ] */
            foreach($arr['fields'] as $k => $v){$arr['address'].=(!empty($this->globals['recipient'][$v]) ? '<span>'.$this->globals['recipient'][$v].'</span><br/>' : '');}
            
            /** [ TOKEN: VALUES ] */
            $tokens['value:date']     =date('j F Y');
            $tokens['value:scompany'] =$this->globals['sender']['company'];
            $tokens['value:logo']     =$this->globals['sender']['logo'];
            $tokens['value:saddress'] =$this->globals['sender']['address'];
            $tokens['value:raddress'] =$arr['address'];
            $tokens['value:content']  =$this->globals['content'];
            
            /** [ TOKEN: CSS ] */
            $tokens['css:pdf_font']         = globals::PDF_FONT;
            $tokens['css:logo_display']     = (!empty($tokens['value:logo']) ? 'block' : 'none');
            $tokens['css:scompany_display'] = (!empty($tokens['value:scompany']) ? 'block' : 'none');
            $tokens['css:saddress_display'] = (!empty($tokens['value:saddress']) ? 'block' : 'none');
            
            /** [ METHOD: LOAD TEMPLATES ] */
            $this->globals['html']=$this->loadTemplate('letter_structure',array(0=>$tokens));

            /** [ RETURN: BOOLEAN ] */
            unset($tokens,$arr); return(true);
            }
            
        /** [ METHOD: CREATE PDF ] */    
        private function createPDF()
            {
            /** [ OBJECT: TCPDF ] */
            $obj=$this->importClass(1);
            
            /** [ SET: HEADER|FOOTER ] */
            $obj->setPrintHeader(false);
            $obj->setPrintFooter(false);
            
            /** [ SET: MARGINS: LEFT|TOP|RIGHT ] */
            $obj->SetMargins(10,0,10,true);
            
            /** [ SET: AUTO PAGE BREAKS ] */
            $obj->SetAutoPageBreak(true);
            
            /** [ SET:FONT ] */
            $obj->SetDefaultMonospacedFont(globals::PDF_FONT);
            $obj->setFontSubsetting(true);
            $obj->SetFont(globals::PDF_FONT,'',12);

            /** [ ADD: PAGE ] */
            $obj->AddPage();
            
            /** [ OUTPUT: HTML CONTENT ] */
            $obj->writeHTML($this->globals['html'],true,false,true,false,'');    

            /** [ METHOD: LAST PAGE ] */
            $obj->lastPage();
            
            /** [ OUTPUT: PDF ] */
            $this->globals['pdf']=$obj->Output('','S');
            
            /** [ RETURN: BOOLEAN ] */
            unset($obj); return(true);
            }
  
        /** [ METHOD: LOAD TEMPLATE ] */
        private function loadTemplate($filename,$tokens)
            {
            /** [ OBJECT: TEMPLATES ] */
            $obj=$this->importClass(2);
            $obj->setFileName($filename);
            $obj->setTokens($tokens);
            $xhtml=$obj->getTemplate();
            
            /** [ RETURN: XHTML ] */
            unset($obj); return($xhtml);
            }       
            
        /** [ METHOD: GET PDF ] */    
        private function getPDFFunc()
            {
            /** [ RETURN: BOOLEAN ] */
            if(empty($this->globals['pdf'])){return(false);}
            
            /** [ RETURN: PDF ] */
            return($this->globals['pdf']);
            }
	    }
?>