<?php
/**	
 *
 *	WHMCS CLASS
 *	Author: Paul Hamlington
 *	Purpose: Control all WHMCS API's from one location. 
 * 
 */

    /** [ SET NAMESPACE: MODULIZED DOCSAWAY ] */
    namespace modulized_docsaway;
    use Illuminate\Database\Capsule\Manager as Capsule;
	

    /** [ WHMCS CLASS ] */
    final class whmcs
        {
		
		
	    /** --------------------------> CLASS INTERFACE METHODS <--------------------------*/
		
        
        /** [ METHOD: SETTINGS ] */
        public function loadSettings($id){return($this->loadSettingsFunc($id));}
        public function saveSettings($id,$content){return($this->saveSettingsFunc($id,$content));}
        public function deleteSettings($id){return($this->deleteSettingsFunc($id));}
        
        /** [ METHOD: CUSTOM FIELDS ] */
        public function createCustomField(){return($this->createCustomFieldFunc());}
        public function deleteCustomField($id){return($this->deleteCustomFieldFunc($id));}
        public function getCustomField($clientID){return($this->getCustomFieldFunc($clientID));}
        
        /** [ METHOD: CREATE TABLES ] */
        public function createTables(){return($this->createTablesFunc());}
        
        /** [ METHOD: FIND ] */
        public function findDetails($id){return($this->findDetailsFunc($id));}        
        
	    /** [ METHOD: GET ] */
        public function getAdmin(){return($this->getAdminFunc());}
        public function getConfiguration(){return($this->getConfigurationFunc());}
	    public function getProducts(){return($this->getProductsFunc());}
        public function getDomainPricing(){return($this->getDomainPricingFunc());}
        public function getDomains($orderID){return($this->getDomainsFunc($orderID));}
        public function getInvoice($id){return($this->getInvoiceFunc($id));}
        public function getClient($id){return($this->getClientFunc($id));}
        public function getContact($id){return($this->getContactFunc($id));}
        public function getContacts($id){return($this->getContactsFunc($id));}
        public function getOrder($id){return($this->getOrderFunc($id));}
        public function getHosting($orderID){return($this->getHostingFunc($orderID));}
        public function getBaseCurrency(){return($this->getBaseCurrencyFunc());}
        
        /** [ METHOD: VERIFICATION ] */
        public function clientVerification($clientID){return($this->clientVerificationFunc($clientID));}  
        public function clientVerificationAdd($clientID,$name,$status,$requestedDate,$code){return($this->clientVerificationAddFunc($clientID,$name,$status,$requestedDate,$code));} 
        public function clientVerificationDelete($clientID){return($this->clientVerificationDeleteFunc($clientID));}
        public function clientVerificationUpdate($id,$completedDate,$status,$reference){return($this->clientVerificationUpdateFunc($id,$completedDate,$status,$reference));}
        public function clientVerificationOverDue($date){return($this->clientVerificationOverDueFunc($date));}
        public function clientVerificationList($status,$page){return($this->clientVerificationListFunc($status,$page));}  
        
        /** [ METHOD: LOG ] */
        public function logActivity($desc){return($this->logActivityFunc($desc));}
        
        /** [ METHOD: PDF ] */
        public function PDFInvoice($id){return($this->PDFInvoiceFunc($id));}        

        
	    /** --------------------------> CLASS FUNCTIONALITY METHODS <--------------------------*/
		
		
        /** [ METHOD: LOAD SETTINGS ] */    
        private function loadSettingsFunc($id)
            {
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($id)){debugger::err('id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('modulized_docsaway_settings')
                ->select('id','value')
                ->where('id','=',$id)
                ->skip(0)
                ->take(1)                        
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('modulized_docsaway_settings query error( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj); return(false);}
            
            /** [ CONVERT: RETURNED ARRAY ] */
            $resultset=array('id'=>$obj[0]->id,'value'=>$obj[0]->value);  
            
            /** [ METHOD: CRYPTION ] */
            $file=globals::cryption($resultset['value'],'decode');

            /** [ RETURN: FILE ] */
            unset($obj,$resultset); return($file);            
            }
            
        /** [ METHOD: SAVE SETTINGS ] */    
        private function saveSettingsFunc($id,$content)  
            {            
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($id)){debugger::err('id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            if(empty($content)){debugger::err('content property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}

            /** [ RUN: DELETE QUERY ] */
            Capsule::table('modulized_docsaway_settings')->where('id','=',$id)->delete();
                        
            /** [ TRY: INSERT QUERY ] */
            try{Capsule::table('modulized_docsaway_settings')->insert(['id'=>$id,'value'=>globals::cryption($content,'encode')]);}
            catch(\Exception $e){debugger::err('modulized_docsaway_settings insert failure( '.$e->getMessage().', id('.$id.') )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}     
             
            /** [ RETURN: BOOLEAN ] */
            return(true);            
            }
            
        /** [ METHOD: DELETE SETTINGS ] */    
        private function deleteSettingsFunc($id)  
            {
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($id)){debugger::err('id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
                        
            /** [ RUN: DELETE QUERY ] */
            try{Capsule::table('modulized_docsaway_settings')->where('id','=',$id)->delete();}
            catch(\Exception $e){debugger::err('modulized_docsaway_settings table setting not deleted( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}

            /** [ RETURN: BOOLEAN ] */
            return(true);            
            }            
                          
        /** [ METHOD: CREATE CUSTOM FIELD ] */     
        private function createCustomFieldFunc()
            {
            /** [ METHOD: LOAD SETTINGS ] */
            $file=$this->loadSettings('config');
        
            /** [ RETURN: CUSTOM FIELD ID ] */
            if($file!==false){return($file['ccField']);}
            
            /** [ METHOD: CHECK CUSTOM FIELD ] */
            if(($id=$this->checkCustomField())!==false){unset($file); return($id);}
                    
            /** [ TRY: INSERT QUERY ] */
            try{$id = Capsule::table('tblcustomfields')->insertGetId(['type'=>'client','fieldname'=>globals::CUSTOM_CLIENT_FIELD_NAME,'description'=>globals::CUSTOM_CLIENT_FIELD_DESC,'fieldtype'=>'tickbox','adminonly'=>'on','sortorder'=>'0']);}
            catch(\Exception $e){debugger::err('Custom field failed during creation( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}     

            /** [ RETURN: ID ] */
            unset($file); return($id);
            }
            
        /** [ METHOD: DELETE CUSTOM FIELD ] */     
        private function deleteCustomFieldFunc($id)
            {
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($id)){debugger::err('id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
                        
            /** [ RUN: DELETE QUERY ] */
            Capsule::table('tblcustomfields')->where('id','=',$id)->delete();            
            
            /** [ RETURN: BOOLEAN ] */
            unset($id); return(true);
            } 
             
        /** [ METHOD: CHECK CUSTOM FIELD ] */     
        private function checkCustomField()
            {
            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('tblcustomfields')
                ->select('id')
                ->where('fieldname','=',globals::CUSTOM_CLIENT_FIELD_NAME)
                ->skip(0)
                ->take(1)                        
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('tblcustomfields query error( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj); return(false);}
            
            /** [ CONVERT: RETURNED ARRAY ] */
            $resultset=array('id'=>$obj[0]->id);  

            /** [ RETURN: ID ] */
            unset($obj); return($resultset['id']);
            }
            
        /** [ METHOD: GET CUSTOM FIELD ] */     
        private function getCustomFieldFunc($clientID)
            {
            /** [ METHOD: LOAD SETTINGS ] */
            $file=$this->loadSettings('config');
            
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($clientID)){debugger::err('client id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            if(empty($file['ccField'])){debugger::err('custom field property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
          
            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('tblcustomfieldsvalues')
                ->select('value')
                ->where('fieldid','=',$file['ccField'])
                ->where('relid','=',$clientID)
                ->skip(0)
                ->take(1)                        
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('tblcustomfieldsvalues query error( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj); return(false);}
            
            /** [ CONVERT: RETURNED ARRAY ] */
            $resultset=array('value'=>$obj[0]->value);  
                  
            /** [ RETURN: RESULTSET ] */
            unset($obj,$file); return($resultset);
            }             
            
        /** [ METHOD: CREATE TABLES ] */
        private function createTablesFunc()
            {
            /** [ CHECK: TABLE EXISTANCE ] */
            try{Capsule::select('DESCRIBE modulized_docsaway_verification'); $arr['table']['tb1']=true;}catch(\Exception $e){$arr['table']['tb1']=false;}
            try{Capsule::select('DESCRIBE modulized_docsaway_settings'); $arr['table']['tb2']=true;}catch(\Exception $e){$arr['table']['tb2']=false;}

            /** [ CREATE: TABLE ] */
            if($arr['table']['tb1']===false)
                {
                /** [ TRY: RUN QUERY ] */
                try
                    {
                    Capsule::schema()
                    ->create('modulized_docsaway_verification',
                    function($table)
                        {
                        $table->engine='MyISAM';
                        $table->integer('client_id')->unsigned();
                        $table->tinyInteger('tbl')->unsigned();
                        $table->string('name',100);
                        $table->date('requested_date');
                        $table->date('due_date');
                        $table->date('completed_date');
                        $table->tinyInteger('status')->unsigned();
                        $table->char('code',6);
                        $table->string('reference',40);
                        $table->primary(array('client_id','tbl'));
                        $table->index(['due_date','status'],'index1');
                        });    
                    } 

                /** [ CATCH: ERROR ] */
                catch(\Exception $e){debugger::err('unable to create modulized_docsaway_verification ('.$e->getMessage().')',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
                }

            /** [ CREATE: TABLE ] */
            if($arr['table']['tb2']===false)
                {                
                /** [ TRY: RUN QUERY ] */
                try
                    {
                    Capsule::schema()
                    ->create('modulized_docsaway_settings',
                    function($table)
                        {
                        $table->engine='MyISAM';
                        $table->string('id',50);
                        $table->text('value');
                        $table->primary('id');
                        });    
                    } 

                /** [ CATCH: ERROR ] */
                catch(\Exception $e){debugger::err('unable to create modulized_docsaway_settings ('.$e->getMessage().')',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
                }

            /** [ RETURN: BOOLEAN ] */
            unset($arr); return(true);           
            }
            
        /** [ METHOD: GET ADMIN ] */     
        private function getAdminFunc()
            {
            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('tbladmins')
                ->select('id','username')
                ->where('roleid','=',1)
                ->orderBy('id', 'asc')
                ->skip(0)
                ->take(1)                        
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('WHMCS admin not loaded( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj); return(false);}
                        
            /** [ CONVERT: RETURNED ARRAY ] */
            $resultset=array('id'=>$obj[0]->id,'username'=>$obj[0]->username);  
                            
            /** [ RETURN: RESULTSET ] */
            unset($obj); return($resultset);
            }            
            
        /** [ METHOD: GET CONFIGURATION ] */     
        private function getConfigurationFunc()
            {
            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('tblconfiguration')
                ->select('setting','value')
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('tblconfiguration query error( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
             
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj); return(false);}
            
            /** [ ERROR: ARRAY NOT SET ] */
            if(!is_array($obj)){unset($obj); return(false);} 
            
            /** [ CONVERT: RETURNED ARRAY ] */
            foreach($obj as $k => $v){$resultset[mb_strtolower($v->setting)]=$v->value;}
            
            /** [ RETURN: RESULTSET ] */
            unset($obj); return($resultset);
            }
 
        /** [ METHOD: GET INVOICE ] */    
        private function getInvoiceFunc($id)
            {            
            /** [ EXTERNAL FUNCTION: LOCAL API ] */
            $file = localAPI('getinvoice',array('invoiceid'=>$id),globals::admin());
            
            /** [ ERROR: FILE NOT FOUND ] */
            if(empty($file)){debugger::err('invoice( '.$id.' ) not found',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($file); return(false);}
            
            /** [ ERROR: WHMCS FEEDBACK ] */
            if($file['status']=='error'){unset($file); return(false);}

            /** [ RETURN: INVOICE DETAILS ] */
            return($file);
            }
              
        /** [ METHOD: GET BASE CURRENCY ] */    
        private function getBaseCurrencyFunc()
            {           
            /** [ EXTERNAL FUNCTION: LOCAL API ] */
            $file = localAPI('getcurrencies','',globals::admin());
            
            /** [ ERROR: FILE NOT FOUND ] */
            if(empty($file)){debugger::err('currencies not found',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($file); return(false);}
            
            /** [ FIND: BASE CURRENCY ] */
            foreach($file['currencies']['currency'] as $k => $v){if($v['rate']==1){unset($file); return($v['code']);}}
            
            /** [ ERROR: BASE CURRENCY NOT FOUND ] */
            debugger::err('base currency not found',__FILE__,__CLASS__,__METHOD__,__LINE__);           
            
            /** [ RETURN: NOTHING ] */
            unset($file); return;
            }            
            
        /** [ METHOD: FIND DETAILS ] */     
        private function findDetailsFunc($id)
            {
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($id)){debugger::err('id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($file); return(false);}
            
            /** [ METHOD: CLIENT ID ] */
            if(($file=globals::clientID($id,false,'DECODE'))===false){unset($file); return(false);}            
           
            /** [ TYPE: SELECTION ] */
            switch($file['type'])
                {
                /** [ METHOD: GET CLIENT ] */
                case 0: $file=$this->getClientFunc($file['id']); break;
                
                /** [ METHOD: GET CONTACT ] */
                case 1: $file=$this->getContactFunc($file['id']); break;
                
                /** [ ERROR: TYPE NOT RECOGNISED ] */
                default: debugger::err('type property not recognised',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);
                }
                   
            /** [ RETURN: FILE ] */
            return($file);
            }
            
        /** [ METHOD: GET CLIENT ] */    
        private function getClientFunc($id)
            { 
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($id)){debugger::err('client id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($file); return(false);}
            
            /** [ EXTERNAL FUNCTION: LOCAL API ] */
            $file = localAPI('getclientsdetails',array('clientid'=>$id),globals::admin());
            
            /** [ ERROR: FILE EMPTY ] */
            if(empty($file)){debugger::err('client( '.$id.' ) not found',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($file); return(false);}
            
            /** [ ERROR: WHMCS FEEDBACK ] */
            if($file['result']=='error'){unset($file); return(false);}            
            
            /** [ RETURN: CLIENT DETAILS ] */
            return($file);
            }
            
        /** [ METHOD: GET CONTACT ] */    
        private function getContactFunc($id)
            {                        
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($id)){debugger::err('contact id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
          
            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('tblcontacts')
                ->select('id','userid','firstname','lastname','companyname','email','address1','address2','city','state','postcode','country','country AS countrycode','phonenumber','subaccount','password')
                ->where('id','=',$id)
                ->skip(0)
                ->take(1)                        
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('WHMCS contact not found( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){debugger::err('contact( '.$id.' ) not found',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($obj); return(false);}
                        
            /** [ CONVERT: RETURNED ARRAY ] */
            $resultset=array('id'           =>$obj[0]->id,
                             'userid'       =>$obj[0]->userid,
                             'firstname'    =>$obj[0]->firstname,
                             'lastname'     =>$obj[0]->lastname,
                             'companyname'  =>$obj[0]->companyname,
                             'email'        =>$obj[0]->email,
                             'address1'     =>$obj[0]->address1,
                             'address2'     =>$obj[0]->address2,
                             'city'         =>$obj[0]->city,
                             'state'        =>$obj[0]->state,
                             'postcode'     =>$obj[0]->postcode,
                             'country'      =>$obj[0]->country,
                             'countrycode'  =>$obj[0]->countrycode,
                             'phonenumber'  =>$obj[0]->phonenumber,
                             'subaccount'   =>$obj[0]->subaccount,
                             'password'     =>$obj[0]->password);  
                                
            /** [ RETURN: RESULTSET ] */
            unset($obj); return($resultset);            
            }             
            
        /** [ METHOD: GET CONTACTS ] */    
        private function getContactsFunc($id)
            { 
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($id)){debugger::err('client id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($file); return(false);}
            
            /** [ EXTERNAL FUNCTION: LOCAL API ] */
            $file = localAPI('getcontacts',array('userid'=>$id),globals::admin());
            
            /** [ ERROR: FILE EMPTY ] */
            if(empty($file)){unset($file); return(false);}
            
            /** [ ERROR: WHMCS FEEDBACK ] */
            if($file['numreturned']==0){unset($file); return(false);}            
            
            /** [ RETURN: CONTACTS DETAILS ] */
            return($file);
            }            
            
        /** [ METHOD: GET ORDER ] */    
        private function getOrderFunc($id)
            {
            /** [ EXTERNAL FUNCTION: LOCAL API ] */
            $file = localAPI('getorders',array('id'=>$id),globals::admin());
            
            /** [ ERROR: FILE EMPTY ] */
            if(empty($file)){debugger::err('order( '.$id.' ) not found',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($file); return(false);}
            
            /** [ ERROR: ORDER ID NOT FOUND ] */
            if($file['orders']['order'][0]['id']!=$id){debugger::err('WHMCS getorders( '.$id.' ) not found )',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($file); return(false);}
            
            /** [ RETURN: ORDER DETAILS ] */
            return($file['orders']['order'][0]);
            } 
            
        /** [ METHOD: GET HOSTING ] */     
        private function getHostingFunc($orderID)
            {
            /** [ ERROR: ORDER ID NOT SET ] */
            if(empty($orderID)){debugger::err('order id not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('tblhosting')
                ->select('orderid','packageid')
                ->where('orderid','=',$orderID)                     
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('tblhosting query failure( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj); return(false);}
                        
            /** [ ERROR: ARRAY NOT SET ] */
            if(!is_array($obj)){unset($obj); return(false);} 
            
            /** [ CONVERT: RETURNED ARRAY ] */
            foreach($obj as $k => $v){$resultset[]=array('orderid'=>$v->orderid,'packageid'=>$v->packageid);}
            
            /** [ RETURN: RESULTSET ] */
            unset($obj); return($resultset);
            } 

        /** [ METHOD: GET PRODUCTS ] */     
        private function getProductsFunc()
            {
            /** [ EXTERNAL FUNCTION: LOCAL API ] */
            $file = localAPI('getproducts','',globals::admin());
   
            /** [ ERROR: FILE EMPTY ] */
            if(empty($file)){unset($file); return(false);}
            
            /** [ BUILD: PRODUCT LIST ] */
            foreach($file['products']['product'] as $k => $v){$resultset[$v['pid']]=$v['name'];}
            
            /** [ SORT ARRAY: MAINTAIN INDEX ASSOCIATION ] */
            asort($resultset);
            
            /** [ RETURN: RESULTSET ] */
            unset($file); return($resultset);
            }
            
        /** [ METHOD: GET DOMAIN PRICING ] */     
        private function getDomainPricingFunc()
            {
            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('tbldomainpricing')
                ->select('id','extension')
                ->orderBy('extension', 'asc')                       
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('tbldomainpricing query failure( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj); return(false);}
                        
            /** [ ERROR: ARRAY NOT SET ] */
            if(!is_array($obj)){unset($obj); return(false);} 
            
            /** [ CONVERT: RETURNED ARRAY ] */
            foreach($obj as $k => $v){$resultset[$v->id]=$v->extension;}
            
            /** [ RETURN: RESULTSET ] */
            unset($obj); return($resultset);
            } 
            
        /** [ METHOD: GET DOMAINS ] */     
        private function getDomainsFunc($orderID)
            {
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($orderID)){debugger::err('order id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}

            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('tbldomains')
                ->select('domain')
                ->where('orderid','=',$orderID)                     
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('tbldomains query failure( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj); return(false);}
                        
            /** [ ERROR: ARRAY NOT SET ] */
            if(!is_array($obj)){unset($obj); return(false);} 
            
            /** [ CONVERT: RETURNED ARRAY ] */
            foreach($obj as $k => $v){$resultset[]=array('domain'=>$v->domain);}
                        
            /** [ RETURN: RESULTSET ] */
            unset($obj); return($resultset);
            }   
         
        /** [ METHOD: CLIENT VERIFICATION ] */     
        private function clientVerificationFunc($id)
            {
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($id)){debugger::err('client id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ METHOD: CLIENT ID ] */
            if(($file=globals::clientID($id,false,'DECODE'))===false){unset($file); return(false);}
            
           /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('modulized_docsaway_verification')
                ->select('client_id','tbl','name','requested_date','due_date','completed_date','status','code','reference')
                ->where('client_id','=',$file['id'])
                ->where('tbl','=',$file['type'])        
                ->skip(0)
                ->take(1)                        
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('modulized_docsaway_verification query failure( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj); return(false);}
                        
            /** [ CONVERT: RETURNED ARRAY ] */
            $resultset=array('client_id'=>$obj[0]->client_id,
                             'tbl'=>$obj[0]->tbl,
                             'name'=>$obj[0]->name,
                             'requested_date'=>$obj[0]->requested_date,
                             'due_date'=>$obj[0]->due_date,
                             'completed_date'=>$obj[0]->completed_date,
                             'status'=>$obj[0]->status,
                             'code'=>$obj[0]->code,
                             'reference'=>$obj[0]->reference);  
                            
            /** [ RETURN: RESULTSET ] */
            unset($obj,$file); return($resultset);           
            }
            
        /** [ METHOD: CLIENT VERIFICATION OVER DUE ] */    
        private function clientVerificationOverDueFunc($date)
            {
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($date)){debugger::err('date property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('modulized_docsaway_verification')
                ->select('client_id','tbl','name','requested_date','due_date','completed_date','status','code','reference')
                ->where('due_date','<',$date)
                ->where('status','=','2')        
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('modulized_docsaway_verification query failure( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj); return(false);}
                        
            /** [ ERROR: ARRAY NOT SET ] */
            if(!is_array($obj)){unset($obj); return(false);} 
            
            /** [ CONVERT: RETURNED ARRAY ] */
            foreach($obj as $k => $v)
                {
                $resultset[]=array('client_id'=>$v->client_id,
                                   'tbl'=>$v->tbl,
                                   'name'=>$v->name,
                                   'requested_date'=>$v->requested_date,
                                   'due_date'=>$v->due_date,
                                   'completed_date'=>$v->completed_date,
                                   'status'=>$v->status,
                                   'code'=>$v->code,
                                   'reference'=>$v->reference);
                }
                                
            
            /** [ TRY: UPDATE QUERY ] */
            try
                {
                Capsule::table('modulized_docsaway_verification')
                ->where('due_date','<',$date)
                ->where('status','=','2')
                ->update(['status'=>'3']);
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('Docsaway address verification update failure( '.$e->getMessage().')',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}     
                         
            /** [ RETURN: RESULTS ] */
            return($resultset);
            }
            
        /** [ METHOD: CLIENT VERIFICATION ADD ] */    
        private function clientVerificationAddFunc($id,$name,$status,$requestedDate,$code)
            {
            /** [ METHOD: CLIENT ID ] */
            if(($file['client']=globals::clientID($id,false,'DECODE'))===false){unset($file); return(false);}
            
            /** [ METHOD: LOAD SETTINGS ] */
            if(($file['cfg']=$this->loadSettings('config'))===false){unset($file); return(false);}
            
            /** [ CALCULATE: REQUESTED|DUE DATES ] */
            $file['date']=explode('-',$requestedDate);
            $file['dueDate']=date('Y-m-d',mktime(0,0,0,$file['date'][1],($file['date'][2]+$file['cfg']['timeFrame']),$file['date'][0]));
            
            /** [ TRY: INSERT QUERY ] */
            try{Capsule::table('modulized_docsaway_verification')->insert(['client_id'=>$file['client']['id'],'tbl'=>$file['client']['type'],'name'=>$name,'requested_date'=>$requestedDate,'due_date'=>$file['dueDate'],'status'=>$status,'code'=>$code]);}
            catch(\Exception $e){debugger::err('Docsaway address verification insert failure( '.$e->getMessage().')',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($file); return(false);}     
                         
            /** [ RETURN: BOOLEAN ] */
            unset($file); return(true);
            }
            
        /** [ METHOD: CLIENT VERIFICATION UPDATE ] */    
        private function clientVerificationUpdateFunc($id,$completedDate,$status,$reference)
            {
            /** [ METHOD: CLIENT ID ] */
            if(($file=globals::clientID($id,false,'DECODE'))===false){unset($file); return(false);}
            
            /** [ WHMCS: UPDATE QUERY ] */
            $arr['values'] = array('completed_date'=>$completedDate,'status'=>$status,'reference'=>$reference);
            foreach($arr['values'] as $k => $v){if($v===false){unset($arr['values'][$k]);}}

            /** [ TRY: UPDATE QUERY ] */
            try
                {
                Capsule::table('modulized_docsaway_verification')
                ->where('client_id','=',$file['id'])
                ->where('tbl','=',$file['type'])
                ->update($arr['values']);
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('Docsaway address verification update failure( '.$e->getMessage().')',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($arr,$file); return(false);}     
                                 
            /** [ RETURN: BOOLEAN ] */
            unset($arr,$file); return(true);
            }            
            
        /** [ METHOD: CLIENT VERIFICATION DELETE ] */     
        private function clientVerificationDeleteFunc($id)
            {
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($id)){debugger::err('client id property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}

            /** [ METHOD: CLIENT ID ] */
            if(($file=globals::clientID($id,false,'DECODE'))===false){unset($file); return(false);}
            
            /** [ TRY: DELETE QUERY ] */
            try{Capsule::table('modulized_docsaway_verification')->where('client_id','=',$file['id'])->where('tbl','=',$file['type'])->delete();}
            catch(\Exception $e){debugger::err('modulized_docsaway_verification delete query failure( '.$e->getMessage().')',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($file); return(false);}          

            /** [ RETURN: BOOLEAN ] */
            unset($file); return(true);
            }             
            
        /** [ METHOD: CLIENT VERIFICATION LIST ] */     
        private function clientVerificationListFunc($status,$page)
            {
            /** [ FORMAT: WHERE CLAUSE ] */
            if($status==0){$arr['like']='%';}else{$arr['like']=$status;}            
            
            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('modulized_docsaway_verification')
                ->select('client_id','tbl','name','requested_date','due_date','completed_date','status','code','reference')
                ->where('status','LIKE',$arr['like'])
                ->orderBy('requested_date','DESC')
                ->skip(($page*globals::PAGING_SIZE))
                ->take(globals::PAGING_SIZE) 
                ->get();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('modulized_docsaway_verification query failure( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($arr); return(array('total'=>0,'resultset'=>false));}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj,$arr); return(array('total'=>0,'resultset'=>false));}
                        
            /** [ ERROR: ARRAY NOT SET ] */
            if(!is_array($obj)){unset($obj,$arr); return(array('total'=>0,'resultset'=>false));} 
            
            /** [ CONVERT: RETURNED ARRAY ] */
            foreach($obj as $k => $v){$resultset[1][]=array('client_id'=>$v->client_id,'tbl'=>$v->tbl,'name'=>$v->name,'requested_date'=>$v->requested_date,'due_date'=>$v->due_date,'completed_date'=>$v->completed_date,'status'=>$v->status,'code'=>$v->code,'reference'=>$v->reference);}
                        
            /** [ TRY: RUN QUERY ] */
            try
                {
                $obj = Capsule::table('modulized_docsaway_verification')
                ->select('client_id')
                ->where('status','LIKE',$arr['like'])
                ->count();
                }
                
            /** [ CATCH: ERROR ] */
            catch(\Exception $e){debugger::err('modulized_docsaway_verification query failure( '.$e->getMessage().' )',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($arr); return(array('total'=>0,'resultset'=>false));}
            
            /** [ ERROR: OBJECT EMPTY ] */
            if(empty($obj)&&!is_numeric($obj)){unset($obj,$arr); return(array('total'=>0,'resultset'=>false));}
                  
            /** [ CONVERT: RETURNED ARRAY ] */
            $resultset[2]=array('total'=>$obj);

            /** [ RETURN: RESULTSET ] */
            unset($obj,$arr); return(array('total'=>$resultset[2]['total'],'resultset'=>$resultset[1]));           
            }            
              
        /** [ METHOD: GET PRODUCTS ] */    
        private function logActivityFunc($desc)
            {
            /** [ ERROR: PROPERTY NOT SET ] */
            if(empty($desc)){debugger::err('description property not set',__FILE__,__CLASS__,__METHOD__,__LINE__); unset($desc); return(false);}
 
            /** [ EXTERNAL FUNCTION: LOCAL API ] */
            localAPI('logactivity',array('description'=>$desc),globals::admin());            
            
            /** [ RETURN: BOOLEAN ] */
            return(true);
            }
            
        /** [ METHOD: PDF INVOICE ] */    
        private function PDFInvoiceFunc($id)
            {
            /** [ ERROR: INVOICE ID NOT SET ] */
            if(empty($id)){debugger::err('PDF invoice id not set',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);}
            
            /** [ LOAD: PDF INVOICE ] */
            ob_start();
            require_once(globals::basePath().'/includes/invoicefunctions.php');                    
            $PDF = pdfinvoice($id);
            ob_end_clean();

            /** [ ERROR: PDF FILE NOT FOUND ] */
            if(empty($PDF)){debugger::err('PDF invoice file( '.$id.' ) not found',__FILE__,__CLASS__,__METHOD__,__LINE__); return(false);} 
                
            /** [ RETURN: PDF ] */    
            return($PDF);
            }              
	    }
?>